/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.auth

import com.kakao.sdk.auth.AuthApiManager
import com.kakao.sdk.auth.model.AccessTokenResponse
import com.kakao.sdk.auth.model.OAuthToken
import com.kakao.sdk.auth.network.kauth
import com.kakao.sdk.common.model.ClientError
import com.kakao.sdk.common.model.ClientErrorCause
import com.kakao.sdk.network.ApiFactory
import retrofit2.Call
import retrofit2.Callback
import retrofit2.HttpException
import retrofit2.Response

private val authApi: PartnerAuthApi
    get() = ApiFactory.kauth.create(PartnerAuthApi::class.java)

/**
 * @suppress
 */
internal fun AuthApiManager.issueAccessToken(
    groupRefreshToken: String,
    callback: (token: OAuthToken?, error: Throwable?) -> Unit
) {
    authApi.issueAccessToken(
        groupRefreshToken,
        clientId = applicationInfo.appKey,
        androidKeyHash = contextInfo.signingKeyHash,
        approvalType = approvalType.value
    )
        .enqueue(object: Callback<AccessTokenResponse> {

            override fun onFailure(call: Call<AccessTokenResponse>, t: Throwable) {
                callback(null, t)
            }

            override fun onResponse(
                call: Call<AccessTokenResponse>,
                response: Response<AccessTokenResponse>
            ) {
                if (response.isSuccessful) {
                    response.body()?.let {
                        val token = OAuthToken.fromResponse(it)
                        tokenManagerProvider.manager.setToken(token)
                        callback(token, null)
                        return
                    }
                    callback(null, ClientError(ClientErrorCause.Unknown, "No body"))
                } else {
                    callback(null, AuthApiManager.translateError(HttpException(response)))
                }
            }
        })
}
